@@ -28,16 +28,6 @@
 
 #include <dbus/dbus-glib.h>
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/enums.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/simple-password-manager.h>
-#include <telepathy-glib/svc-connection.h>
-#include <telepathy-glib/channel-manager.h>
-#include <telepathy-glib/gtypes.h>
-#include <telepathy-glib/util.h>
-
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_CONNECTION
 #include "idle-contact-info.h"
 #include "idle-ctcp.h"
@@ -48,6 +38,7 @@
 #include "idle-roomlist-manager.h"
 #include "idle-parser.h"
 #include "idle-server-connection.h"
+#include "server-tls-manager.h"
 
 #include "extensions/extensions.h"    /* Renaming */
 
@@ -145,7 +136,14 @@ struct _IdleConnectionPrivate {
 	 * network connection
 	 */
 	IdleServerConnection *conn;
-	guint sconn_status;
+	/*
+	 * TRUE if 'conn' is connected to the server. (This just represents the TCP
+	 * stream; not whether we are authenticated.)
+	 */
+	gboolean sconn_connected;
+
+	/* Used for idle_server_connection_connect_async(). */
+	GCancellable *connect_cancellable;
 
 	/* When we sent a PING to the server which it hasn't PONGed for yet, or 0 if
 	 * there isn't a PING outstanding.
@@ -199,6 +197,12 @@ struct _IdleConnectionPrivate {
 
 	/* so we can pop up a SASL channel asking for the password */
 	TpSimplePasswordManager *password_manager;
+
+	/* TLS channel */
+	IdleServerTLSManager *tls_manager;
+
+	/* TpHandle -> owned gchar * */
+	GHashTable *aliases;
 };
 
 static void _iface_create_handle_repos(TpBaseConnection *self, TpHandleRepoIface **repos);
@@ -219,7 +223,7 @@ static IdleParserHandlerResult _version_privmsg_handler(IdleParser *parser, Idle
 static IdleParserHandlerResult _welcome_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _whois_user_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 
-static void sconn_status_changed_cb(IdleServerConnection *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn);
+static void sconn_disconnected_cb(IdleServerConnection *sconn, IdleServerConnectionStateReason reason, IdleConnection *conn);
 static void sconn_received_cb(IdleServerConnection *sconn, gchar *raw_msg, IdleConnection *conn);
 
 static void irc_handshakes(IdleConnection *conn);
@@ -242,8 +246,9 @@ static void idle_connection_init(IdleConnection *obj) {
 	IdleConnectionPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (obj, IDLE_TYPE_CONNECTION, IdleConnectionPrivate);
 
 	obj->priv = priv;
-	priv->sconn_status = SERVER_CONNECTION_STATE_NOT_CONNECTED;
+	priv->sconn_connected = FALSE;
 	priv->msg_queue = g_queue_new();
+	priv->aliases = g_hash_table_new_full (NULL, NULL, NULL, g_free);
 
 	tp_contacts_mixin_init ((GObject *) obj, G_STRUCT_OFFSET (IdleConnection, contacts));
 	tp_base_connection_register_with_contacts_mixin ((TpBaseConnection *) obj);
@@ -400,6 +405,8 @@ static void idle_connection_dispose (GObject *object) {
 		priv->conn = NULL;
 	}
 
+	g_clear_object (&priv->connect_cancellable);
+
 	if (priv->queued_aliases_owners)
 		tp_handle_set_destroy(priv->queued_aliases_owners);
 
@@ -408,6 +415,8 @@ static void idle_connection_dispose (GObject *object) {
 
 	g_object_unref(self->parser);
 
+	tp_clear_pointer (&priv->aliases, g_hash_table_unref);
+
 	if (G_OBJECT_CLASS(idle_connection_parent_class)->dispose)
 		G_OBJECT_CLASS(idle_connection_parent_class)->dispose (object);
 }
@@ -450,6 +459,21 @@ const gchar * const *idle_connection_get_implemented_interfaces (void) {
 	return interfaces_always_present;
 }
 
+static GPtrArray *
+get_interfaces_always_present (TpBaseConnection *base)
+{
+	GPtrArray *interfaces;
+	const gchar **iter;
+
+	interfaces = TP_BASE_CONNECTION_CLASS (
+		idle_connection_parent_class)->get_interfaces_always_present (base);
+
+	for (iter = interfaces_always_present; *iter != NULL; iter++)
+		g_ptr_array_add (interfaces, (gchar *) *iter);
+
+	return interfaces;
+}
+
 static void idle_connection_class_init(IdleConnectionClass *klass) {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	TpBaseConnectionClass *parent_class = TP_BASE_CONNECTION_CLASS(klass);
@@ -472,7 +496,7 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	parent_class->disconnected = _iface_disconnected;
 	parent_class->shut_down = _iface_shut_down;
 	parent_class->start_connecting = _iface_start_connecting;
-	parent_class->interfaces_always_present = interfaces_always_present;
+	parent_class->get_interfaces_always_present = get_interfaces_always_present;
 
 	param_spec = g_param_spec_string("nickname", "IRC nickname", "The nickname to be visible to others in IRC.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_NICKNAME, param_spec);
@@ -492,7 +516,7 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	param_spec = g_param_spec_string("username", "User name", "The username of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_USERNAME, param_spec);
 
-	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", "NULL", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
+	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_CHARSET, param_spec);
 
 	param_spec = g_param_spec_uint("keepalive-interval", "Keepalive interval", "Seconds between keepalive packets, or 0 to disable", 0, G_MAXUINT, DEFAULT_KEEPALIVE_INTERVAL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
@@ -533,6 +557,11 @@ static GPtrArray *_iface_create_channel_managers(TpBaseConnection *base) {
 	manager = g_object_new(IDLE_TYPE_ROOMLIST_MANAGER, "connection", self, NULL);
 	g_ptr_array_add(managers, manager);
 
+	priv->tls_manager = g_object_new (IDLE_TYPE_SERVER_TLS_MANAGER,
+		"connection", self,
+                NULL);
+	g_ptr_array_add(managers, priv->tls_manager);
+
 	return managers;
 }
 
@@ -581,7 +610,7 @@ static void _iface_disconnected(TpBaseConnection *self) {
 	/* we never got around to actually creating the connection
 	 * iface object because we were still trying to connect, so
 	 * don't try to send any traffic down it */
-	if (priv->conn == NULL) {
+	if (!priv->sconn_connected) {
 		return;
 	}
 
@@ -607,13 +636,16 @@ static void _iface_shut_down(TpBaseConnection *base) {
 	 * don't try to send any traffic down it */
 	if (priv->conn == NULL) {
 		g_idle_add(_finish_shutdown_idle_func, self);
+	} else if (!priv->sconn_connected) {
+		IDLE_DEBUG("cancelling connection");
+		g_cancellable_cancel (priv->connect_cancellable);
 	} else {
 		idle_server_connection_disconnect_async(priv->conn, NULL, NULL, NULL);
 	}
 }
 
 static void _connection_disconnect_with_gerror(IdleConnection *conn, TpConnectionStatusReason reason, const gchar *key, const GError *error) {
-	if (TP_BASE_CONNECTION (conn)->status == TP_CONNECTION_STATUS_DISCONNECTED) {
+	if (tp_base_connection_get_status (TP_BASE_CONNECTION (conn)) == TP_CONNECTION_STATUS_DISCONNECTED) {
 		IDLE_DEBUG ("Already disconnected; refusing to report error %s", error->message);
 	} else {
 		GHashTable *details = tp_asv_new(key, G_TYPE_STRING, error->message, NULL);
@@ -649,7 +681,7 @@ static void _password_prompt_cb(GObject *source, GAsyncResult *result, gpointer
 	if (error != NULL) {
 		IDLE_DEBUG("Simple password manager failed: %s", error->message);
 
-		if (base_conn->status != TP_CONNECTION_STATUS_DISCONNECTED)
+		if (tp_base_connection_get_status (base_conn) != TP_CONNECTION_STATUS_DISCONNECTED)
 			_connection_disconnect_with_gerror(conn,
 							   TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED,
 							   "debug-message",
@@ -698,11 +730,10 @@ static void _connection_connect_ready(GObject *source_object, GAsyncResult *res,
 		IDLE_DEBUG("idle_server_connection_connect failed: %s", error->message);
 		_connection_disconnect_with_gerror(conn, TP_CONNECTION_STATUS_REASON_NETWORK_ERROR, "debug-message", error);
 		g_error_free(error);
-		g_object_unref(sconn);
 		return;
 	}
 
-	priv->conn = sconn;
+	priv->sconn_connected = TRUE;
 
 	g_signal_connect(sconn, "received", (GCallback)(sconn_received_cb), conn);
 
@@ -742,30 +773,34 @@ static void _start_connecting_continue(IdleConnection *conn) {
 		priv->username = g_strdup(g_get_user_name());
 	}
 
-	sconn = g_object_new(IDLE_TYPE_SERVER_CONNECTION, "host", priv->server, "port", priv->port, NULL);
+	sconn = g_object_new(IDLE_TYPE_SERVER_CONNECTION,
+            "host", priv->server,
+            "port", priv->port,
+            "tls-manager", priv->tls_manager,
+            NULL);
 	if (priv->use_ssl)
 		idle_server_connection_set_tls(sconn, TRUE);
 
-	g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
+	g_signal_connect(sconn, "disconnected", (GCallback)(sconn_disconnected_cb), conn);
 
-	idle_server_connection_connect_async(sconn, NULL, _connection_connect_ready, conn);
+	priv->conn = sconn;
+	g_warn_if_fail (priv->connect_cancellable == NULL);
+	priv->connect_cancellable = g_cancellable_new ();
+	idle_server_connection_connect_async(sconn, priv->connect_cancellable, _connection_connect_ready, conn);
 }
 
 static gboolean keepalive_timeout_cb(gpointer user_data);
 
-static void sconn_status_changed_cb(IdleServerConnection *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn) {
+static void sconn_disconnected_cb(IdleServerConnection *sconn, IdleServerConnectionStateReason reason, IdleConnection *conn) {
 	IdleConnectionPrivate *priv = conn->priv;
 	TpConnectionStatusReason tp_reason;
 
 	/* cancel scheduled forced disconnect since we are now disconnected */
-	if (state == SERVER_CONNECTION_STATE_NOT_CONNECTED &&
-		priv->force_disconnect_id) {
+	if (priv->force_disconnect_id) {
 		g_source_remove(priv->force_disconnect_id);
 		priv->force_disconnect_id = 0;
 	}
 
-	IDLE_DEBUG("called with state %u", state);
-
 	switch (reason) {
 		case SERVER_CONNECTION_STATE_REASON_ERROR:
 			tp_reason = TP_CONNECTION_STATUS_REASON_NETWORK_ERROR;
@@ -783,30 +818,8 @@ static void sconn_status_changed_cb(IdleServerConnection *sconn, IdleServerConne
 	if (priv->quitting)
 		tp_reason = TP_CONNECTION_STATUS_REASON_REQUESTED;
 
-	switch (state) {
-		case SERVER_CONNECTION_STATE_NOT_CONNECTED:
-			connection_disconnect_cb(conn, tp_reason);
-			break;
-
-		case SERVER_CONNECTION_STATE_CONNECTING:
-			break;
-
-		case SERVER_CONNECTION_STATE_CONNECTED:
-			if (priv->keepalive_interval != 0 && priv->keepalive_timeout == 0)
-				priv->keepalive_timeout = g_timeout_add_seconds(priv->keepalive_interval, keepalive_timeout_cb, conn);
-
-			if (g_queue_get_length(priv->msg_queue) > 0) {
-				IDLE_DEBUG("we had messages in queue, start unloading them now");
-				idle_connection_add_queue_timeout (conn);
-			}
-			break;
-
-		default:
-			g_assert_not_reached();
-			break;
-	}
-
-	priv->sconn_status = state;
+	priv->sconn_connected = FALSE;
+	connection_disconnect_cb(conn, tp_reason);
 }
 
 static void sconn_received_cb(IdleServerConnection *sconn, gchar *raw_msg, IdleConnection *conn) {
@@ -822,7 +835,7 @@ static gboolean keepalive_timeout_cb(gpointer user_data) {
 	gchar cmd[IRC_MSG_MAXLEN + 1];
 	gint64 now;
 
-	if (priv->sconn_status != SERVER_CONNECTION_STATE_CONNECTED ||
+	if (!priv->sconn_connected ||
 	    priv->quitting) {
 		priv->keepalive_timeout = 0;
 		return FALSE;
@@ -882,7 +895,7 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 
 	IDLE_DEBUG("called");
 
-	if (priv->sconn_status != SERVER_CONNECTION_STATE_CONNECTED) {
+	if (!priv->sconn_connected) {
 		IDLE_DEBUG("connection was not connected!");
 
 		priv->msg_queue_timeout = 0;
@@ -1006,7 +1019,7 @@ idle_connection_get_max_message_length(IdleConnection *conn)
 
 static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
-	TpConnectionStatus status = conn->parent.status;
+	TpConnectionStatus status = tp_base_connection_get_status (TP_BASE_CONNECTION (conn));
 	TpConnectionStatusReason reason;
 	const gchar *msg;
 	const gchar *begin;
@@ -1051,7 +1064,7 @@ static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMess
 static IdleParserHandlerResult _erroneous_nickname_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 
-	if (conn->parent.status == TP_CONNECTION_STATUS_CONNECTING)
+	if (tp_base_connection_get_status (TP_BASE_CONNECTION (conn)) == TP_CONNECTION_STATUS_CONNECTING)
 		connection_connect_cb(conn, FALSE, TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -1065,7 +1078,7 @@ static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessa
 	if (old_handle == new_handle)
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 
-	if (old_handle == conn->parent.self_handle) {
+	if (old_handle == tp_base_connection_get_self_handle (TP_BASE_CONNECTION (conn))) {
 		IDLE_DEBUG("Self renamed: handle was %d, now %d", old_handle, new_handle);
 		tp_base_connection_set_self_handle(TP_BASE_CONNECTION(conn), new_handle);
 	}
@@ -1080,7 +1093,7 @@ static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessa
 static IdleParserHandlerResult _nickname_in_use_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 
-	if (conn->parent.status == TP_CONNECTION_STATUS_CONNECTING)
+	if (tp_base_connection_get_status (TP_BASE_CONNECTION (conn)) == TP_CONNECTION_STATUS_CONNECTING)
 		connection_connect_cb(conn, FALSE, TP_CONNECTION_STATUS_REASON_NAME_IN_USE);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -1219,17 +1232,27 @@ static void send_quit_request(IdleConnection *conn) {
 
 static void connection_connect_cb(IdleConnection *conn, gboolean success, TpConnectionStatusReason fail_reason) {
 	TpBaseConnection *base = TP_BASE_CONNECTION(conn);
+	IdleConnectionPrivate *priv = conn->priv;
 
-	if (success)
+	if (success) {
 		tp_base_connection_change_status(base, TP_CONNECTION_STATUS_CONNECTED, TP_CONNECTION_STATUS_REASON_REQUESTED);
-	else
+
+		if (priv->keepalive_interval != 0 && priv->keepalive_timeout == 0)
+			priv->keepalive_timeout = g_timeout_add_seconds(priv->keepalive_interval, keepalive_timeout_cb, conn);
+
+		if (g_queue_get_length(priv->msg_queue) > 0) {
+			IDLE_DEBUG("we had messages in queue, start unloading them now");
+			idle_connection_add_queue_timeout (conn);
+		}
+	} else {
 		tp_base_connection_change_status(base, TP_CONNECTION_STATUS_DISCONNECTED, fail_reason);
+	}
 }
 
 static void connection_disconnect_cb(IdleConnection *conn, TpConnectionStatusReason reason) {
 	TpBaseConnection *base = TP_BASE_CONNECTION(conn);
 
-	if (base->status == TP_CONNECTION_STATUS_DISCONNECTED)
+	if (tp_base_connection_get_status (base) == TP_CONNECTION_STATUS_DISCONNECTED)
 		g_idle_add(_finish_shutdown_idle_func, base);
 	else
 		tp_base_connection_change_status(base, TP_CONNECTION_STATUS_DISCONNECTED, reason);
@@ -1258,19 +1281,9 @@ _queue_alias_changed(IdleConnection *conn, TpHandle handle, const gchar *alias)
 			G_TYPE_INVALID));
 }
 
-static GQuark
-_canon_nick_quark (void) {
-	static GQuark quark = 0;
-
-	if (!quark)
-		quark = g_quark_from_static_string("canon-nick");
-
-	return quark;
-}
-
 void idle_connection_canon_nick_receive(IdleConnection *conn, TpHandle handle, const gchar *canon_nick) {
 	TpHandleRepoIface *handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(conn), TP_HANDLE_TYPE_CONTACT);
-	const gchar *old_alias = tp_handle_get_qdata(handles, handle, _canon_nick_quark());
+	const gchar *old_alias = g_hash_table_lookup (conn->priv->aliases, GUINT_TO_POINTER (handle));
 
 	if (!old_alias)
 		old_alias = tp_handle_inspect(handles, handle);
@@ -1278,7 +1291,7 @@ void idle_connection_canon_nick_receive(IdleConnection *conn, TpHandle handle, c
 	if (!strcmp(old_alias, canon_nick))
 		return;
 
-	tp_handle_set_qdata(handles, handle, _canon_nick_quark(), g_strdup(canon_nick), g_free);
+	g_hash_table_insert (conn->priv->aliases, GUINT_TO_POINTER (handle), g_strdup (canon_nick));
 
 	_queue_alias_changed(conn, handle, canon_nick);
 }
@@ -1304,11 +1317,12 @@ static void idle_connection_get_alias_flags(TpSvcConnectionInterfaceAliasing *if
 }
 
 static const gchar *
-gimme_an_alias (
+gimme_an_alias (IdleConnection *self,
     TpHandleRepoIface *repo,
     TpHandle handle)
 {
-  const gchar *alias = tp_handle_get_qdata (repo, handle, _canon_nick_quark());
+  const gchar *alias = g_hash_table_lookup (self->priv->aliases,
+      GUINT_TO_POINTER (handle));
 
   if (alias != NULL)
     return alias;
@@ -1330,7 +1344,7 @@ conn_aliasing_fill_contact_attributes (
   for (i = 0; i < contacts->len; i++)
     {
       TpHandle handle = g_array_index (contacts, TpHandle, i);
-      const gchar *alias = gimme_an_alias (repo, handle);
+      const gchar *alias = gimme_an_alias (self, repo, handle);
 
       g_assert (alias != NULL);
 
@@ -1346,6 +1360,7 @@ idle_connection_get_aliases (
     const GArray *handles,
     DBusGMethodInvocation *context)
 {
+  IdleConnection *self = IDLE_CONNECTION (iface);
   TpHandleRepoIface *repo = tp_base_connection_get_handles (
       TP_BASE_CONNECTION (iface), TP_HANDLE_TYPE_CONTACT);
   GError *error = NULL;
@@ -1365,7 +1380,7 @@ idle_connection_get_aliases (
       TpHandle handle = g_array_index (handles, TpHandle, i);
 
       g_hash_table_insert (aliases, GUINT_TO_POINTER (handle),
-          (gpointer) gimme_an_alias (repo, handle));
+          (gpointer) gimme_an_alias (self, repo, handle));
     }
 
   tp_svc_connection_interface_aliasing_return_from_get_aliases (context,
@@ -1374,6 +1389,7 @@ idle_connection_get_aliases (
 }
 
 static void idle_connection_request_aliases(TpSvcConnectionInterfaceAliasing *iface, const GArray *handles, DBusGMethodInvocation *context) {
+	IdleConnection *self = IDLE_CONNECTION (iface);
 	TpHandleRepoIface *repo = tp_base_connection_get_handles(TP_BASE_CONNECTION(iface), TP_HANDLE_TYPE_CONTACT);
 	GError *error = NULL;
 	const gchar **aliases;
@@ -1388,7 +1404,7 @@ static void idle_connection_request_aliases(TpSvcConnectionInterfaceAliasing *if
 	for (guint i = 0; i < handles->len; i++) {
 		TpHandle handle = g_array_index(handles, TpHandle, i);
 
-		aliases[i] = gimme_an_alias (repo, handle);
+		aliases[i] = gimme_an_alias (self, repo, handle);
 	}
 
 	tp_svc_connection_interface_aliasing_return_from_request_aliases(context, aliases);
@@ -1424,7 +1440,7 @@ static void idle_connection_request_rename(IdleSvcConnectionInterfaceRenaming *i
 
 static void idle_connection_set_aliases(TpSvcConnectionInterfaceAliasing *iface, GHashTable *aliases, DBusGMethodInvocation *context) {
 	IdleConnection *conn = IDLE_CONNECTION(iface);
-	const gchar *requested_alias = g_hash_table_lookup(aliases, GUINT_TO_POINTER(conn->parent.self_handle));
+	const gchar *requested_alias = g_hash_table_lookup(aliases, GUINT_TO_POINTER(tp_base_connection_get_self_handle (TP_BASE_CONNECTION (conn))));
 
 	if ((g_hash_table_size(aliases) != 1) || !requested_alias) {
 		GError error = {TP_ERROR, TP_ERROR_NOT_AVAILABLE, "You can only set your own alias in IRC"};
