@@ -29,14 +29,7 @@
 
 #include <dbus/dbus-glib.h>
 
-#include <telepathy-glib/channel-iface.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/enums.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/gtypes.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/handle.h>
-#include <telepathy-glib/svc-channel.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_MUC
 #include "idle-connection.h"
@@ -46,6 +39,7 @@
 
 static void _password_iface_init(gpointer, gpointer);
 static void subject_iface_init(gpointer, gpointer);
+static void destroyable_iface_init(gpointer, gpointer);
 
 static void idle_muc_channel_send (GObject *obj, TpMessage *message, TpMessageSendingFlags flags);
 static void idle_muc_channel_close (TpBaseChannel *base);
@@ -59,6 +53,7 @@ G_DEFINE_TYPE_WITH_CODE(IdleMUCChannel, idle_muc_channel, TP_TYPE_BASE_CHANNEL,
 		G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_SUBJECT, subject_iface_init);
 		G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_ROOM_CONFIG,
                                        tp_base_room_config_iface_init);
+		G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_DESTROYABLE, destroyable_iface_init);
 		)
 
 /* property enum */
@@ -131,6 +126,7 @@ static const gchar *muc_channel_interfaces[] = {
 	TP_IFACE_CHANNEL_INTERFACE_ROOM,
 	TP_IFACE_CHANNEL_INTERFACE_SUBJECT,
 	TP_IFACE_CHANNEL_INTERFACE_ROOM_CONFIG,
+	TP_IFACE_CHANNEL_INTERFACE_DESTROYABLE,
 	NULL
 };
 
@@ -209,6 +205,7 @@ idle_muc_channel_constructed (GObject *obj)
 			"text/plain",
 			NULL
 	};
+	TpHandle self_handle = tp_base_connection_get_self_handle (conn);
 
 	G_OBJECT_CLASS (idle_muc_channel_parent_class)->constructed (obj);
 
@@ -217,7 +214,7 @@ idle_muc_channel_constructed (GObject *obj)
 
 	tp_base_channel_register (base);
 
-	tp_group_mixin_init(obj, G_STRUCT_OFFSET(IdleMUCChannel, group), contact_handles, conn->self_handle);
+	tp_group_mixin_init(obj, G_STRUCT_OFFSET(IdleMUCChannel, group), contact_handles, self_handle);
 	tp_group_mixin_change_flags(obj, TP_CHANNEL_GROUP_FLAG_PROPERTIES, 0);
 
 	/* initialize message mixin */
@@ -233,7 +230,7 @@ idle_muc_channel_constructed (GObject *obj)
 		TpIntset *remote;
 		TpHandle initiator = tp_base_channel_get_initiator (base);
 
-		g_assert (initiator == conn->self_handle);
+		g_assert (initiator == self_handle);
 
 		remote = tp_intset_new_containing (initiator);
 		tp_group_mixin_change_members (obj, "", NULL, NULL, NULL, remote,
@@ -309,6 +306,20 @@ idle_muc_channel_fill_immutable_properties (
       NULL);
 }
 
+static GPtrArray *
+idle_muc_channel_get_interfaces (TpBaseChannel *self)
+{
+  TpBaseChannelClass *parent_class =
+      TP_BASE_CHANNEL_CLASS (idle_muc_channel_parent_class);
+  GPtrArray *interfaces = parent_class->get_interfaces (self);
+  const gchar **interface;
+
+  for (interface = muc_channel_interfaces; *interface != NULL; interface++)
+    g_ptr_array_add (interfaces, (gchar *) *interface);
+
+  return interfaces;
+}
+
 static void idle_muc_channel_class_init (IdleMUCChannelClass *idle_muc_channel_class) {
 	GObjectClass *object_class = G_OBJECT_CLASS (idle_muc_channel_class);
 	TpBaseChannelClass *base_channel_class = TP_BASE_CHANNEL_CLASS (idle_muc_channel_class);
@@ -336,11 +347,11 @@ static void idle_muc_channel_class_init (IdleMUCChannelClass *idle_muc_channel_c
 
 	base_channel_class->channel_type = TP_IFACE_CHANNEL_TYPE_TEXT;
 	base_channel_class->target_handle_type = TP_HANDLE_TYPE_ROOM;
-	base_channel_class->interfaces = muc_channel_interfaces;
 
 	base_channel_class->close = idle_muc_channel_close;
 	base_channel_class->fill_immutable_properties = idle_muc_channel_fill_immutable_properties;
 	base_channel_class->get_object_path_suffix = idle_muc_channel_get_path_suffix;
+	base_channel_class->get_interfaces = idle_muc_channel_get_interfaces;
 
 	param_spec = g_param_spec_string (
 		"server", "Room.Server", "always empty",
@@ -727,7 +738,7 @@ void idle_muc_channel_join(IdleMUCChannel *chan, TpHandle joiner) {
 	set = tp_intset_new();
 	tp_intset_add(set, joiner);
 
-	if (joiner == base_conn->self_handle) {
+	if (joiner == tp_base_connection_get_self_handle (base_conn)) {
 		/* woot we managed to get into a channel, great */
 		change_state(chan, MUC_STATE_JOINED);
 		tp_group_mixin_change_members((GObject *)(chan), NULL, set, NULL, NULL, NULL, joiner, TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
@@ -758,7 +769,7 @@ static void _network_member_left(IdleMUCChannel *chan, TpHandle leaver, TpHandle
 	tp_intset_add(set, leaver);
 	tp_group_mixin_change_members((GObject *) chan, message, NULL, set, NULL, NULL, actor, reason);
 
-	if (leaver == base_conn->self_handle) {
+	if (leaver == tp_base_connection_get_self_handle (base_conn)) {
 		change_state(chan, MUC_STATE_PARTED);
 
 		if (!tp_base_channel_is_destroyed (base)) {
@@ -788,7 +799,7 @@ void idle_muc_channel_invited(IdleMUCChannel *chan, TpHandle inviter) {
 	TpIntset *local = tp_intset_new();
 
 	tp_intset_add(add, inviter);
-	tp_intset_add(local, base_conn->self_handle);
+	tp_intset_add(local, tp_base_connection_get_self_handle (base_conn));
 
 	tp_group_mixin_change_members((GObject *)(chan), NULL, add, NULL, local, NULL, inviter, TP_CHANNEL_GROUP_CHANGE_REASON_INVITED);
 
@@ -812,7 +823,7 @@ void idle_muc_channel_namereply(IdleMUCChannel *chan, GValueArray *args) {
 		gchar modechar = g_value_get_char(g_value_array_get_nth(args, i + 1));
 #endif
 
-		if (handle == base_conn->self_handle) {
+		if (handle == tp_base_connection_get_self_handle (base_conn)) {
 			guint remove = MODE_FLAG_OPERATOR_PRIVILEGE | MODE_FLAG_VOICE_PRIVILEGE | MODE_FLAG_HALFOP_PRIVILEGE;
 			guint add = 0;
 
@@ -898,7 +909,7 @@ void idle_muc_channel_mode(IdleMUCChannel *chan, GValueArray *args) {
 					if ((i + 1) < args->n_values) {
 						TpHandle handle = tp_handle_ensure(handles, g_value_get_string(g_value_array_get_nth(args, ++i)), NULL, NULL);
 
-						if (handle == base_conn->self_handle) {
+						if (handle == tp_base_connection_get_self_handle (base_conn)) {
 							IDLE_DEBUG("got MODE '%c' concerning us", *modes);
 							mode_accum |= _modechar_to_modeflag(*modes);
 						}
@@ -1224,8 +1235,9 @@ static gboolean add_member(GObject *gobj, TpHandle handle, const gchar *message,
 	IdleMUCChannelPrivate *priv = obj->priv;
 	TpBaseChannel *base = TP_BASE_CHANNEL (obj);
 	TpBaseConnection *base_conn = tp_base_channel_get_connection (base);
+	TpHandle self_handle = tp_base_connection_get_self_handle (base_conn);
 
-	if (handle == base_conn->self_handle) {
+	if (handle == self_handle) {
 		if (tp_handle_set_is_member(obj->group.members, handle) || tp_handle_set_is_member(obj->group.remote_pending, handle)) {
 			GError *e = g_error_new (TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 				"we are already a member of or trying to join %s", priv->channel_name);
@@ -1265,7 +1277,7 @@ static gboolean add_member(GObject *gobj, TpHandle handle, const gchar *message,
 
 			tp_intset_add(add_set, handle);
 
-			tp_group_mixin_change_members(gobj, NULL, NULL, NULL, NULL, add_set, base_conn->self_handle, TP_CHANNEL_GROUP_CHANGE_REASON_INVITED);
+			tp_group_mixin_change_members(gobj, NULL, NULL, NULL, NULL, add_set, self_handle, TP_CHANNEL_GROUP_CHANGE_REASON_INVITED);
 		}
 	}
 
@@ -1295,7 +1307,7 @@ static gboolean remove_member(GObject *gobj, TpHandle handle, const gchar *messa
 	TpBaseChannel *base = TP_BASE_CHANNEL (obj);
 	TpBaseConnection *base_conn = tp_base_channel_get_connection (base);
 
-	if (handle == base_conn->self_handle) {
+	if (handle == tp_base_connection_get_self_handle (base_conn)) {
 		part_from_channel(obj, message);
 		return TRUE;
 	}
@@ -1325,26 +1337,38 @@ idle_muc_channel_close (
 
 	IDLE_DEBUG ("called on %p", self);
 
+	if (priv->state == MUC_STATE_JOINED) {
+		tp_message_mixin_set_rescued (G_OBJECT (self));
+		tp_base_channel_reopened (base, 0);
+	} else {
+	/* FIXME: this is wrong if called while JOIN is in flight. */
+		tp_base_channel_destroyed (base);
+	}
+}
+
+
+static void
+idle_muc_channel_destroy (
+    TpSvcChannelInterfaceDestroyable *object,
+    DBusGMethodInvocation *context)
+{
+	TpBaseChannel *base = TP_BASE_CHANNEL (object);
+	IdleMUCChannel *self = IDLE_MUC_CHANNEL (object);
+	IdleMUCChannelPrivate *priv = self->priv;
+
+	IDLE_DEBUG ("called on %p", self);
+
 	if (priv->state == MUC_STATE_JOINED)
 		part_from_channel (self, NULL);
 
 	/* FIXME: this is wrong if called while JOIN is in flight. */
 	if (priv->state < MUC_STATE_JOINED)
 		tp_base_channel_destroyed (base);
+
+	tp_svc_channel_interface_destroyable_return_from_destroy (context);
 }
 
-/**
- * idle_muc_channel_get_password_flags
- *
- * Implements DBus method GetPasswordFlags
- * on interface org.freedesktop.Telepathy.Channel.Interface.Password
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
+
 static void idle_muc_channel_get_password_flags (TpSvcChannelInterfacePassword *iface, DBusGMethodInvocation *context) {
 	IdleMUCChannel *obj = IDLE_MUC_CHANNEL(iface);
 	IdleMUCChannelPrivate *priv;
@@ -1358,18 +1382,6 @@ static void idle_muc_channel_get_password_flags (TpSvcChannelInterfacePassword *
 }
 
 
-/**
- * idle_muc_channel_provide_password
- *
- * Implements DBus method ProvidePassword
- * on interface org.freedesktop.Telepathy.Channel.Interface.Password
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
- */
 static void idle_muc_channel_provide_password (TpSvcChannelInterfacePassword *iface, const gchar * password, DBusGMethodInvocation *context) {
 	IdleMUCChannel *obj = IDLE_MUC_CHANNEL(iface);
 	IdleMUCChannelPrivate *priv;
@@ -1523,3 +1535,16 @@ subject_iface_init (
   IMPLEMENT (set_subject);
 #undef IMPLEMENT
 }
+
+static void
+destroyable_iface_init (
+    gpointer g_iface,
+    gpointer iface_data G_GNUC_UNUSED)
+{
+  TpSvcChannelInterfaceDestroyableClass *klass = g_iface;
+
+#define IMPLEMENT(x) \
+  tp_svc_channel_interface_destroyable_implement_##x (klass, idle_muc_channel_##x)
+  IMPLEMENT (destroy);
+#undef IMPLEMENT
+}
