@@ -24,9 +24,7 @@
 
 #include <time.h>
 
-#include <telepathy-glib/channel-manager.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_MUC
 #include "idle-connection.h"
@@ -272,7 +270,7 @@ static IdleParserHandlerResult _invite_handler(IdleParser *parser, IdleParserMes
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 2));
 	IdleMUCChannel *chan;
 
-	if (invited_handle != priv->conn->parent.self_handle)
+	if (invited_handle != tp_base_connection_get_self_handle (TP_BASE_CONNECTION (priv->conn)))
 		return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 
 	if (!priv->channels) {
@@ -741,7 +739,7 @@ static void _channel_join_ready_cb(IdleMUCChannel *chan, guint err, gpointer use
 			break;
 	}
 
-	for (l = reqs; reqs != NULL; reqs = reqs->next) {
+	for (l = reqs; l != NULL; l = l->next) {
 		tp_channel_manager_emit_request_failed(manager, l->data, TP_ERROR, err_code, err_msg);
 	}
 
@@ -832,7 +830,8 @@ _muc_manager_request (
     }
   else
     {
-      channel = _muc_manager_new_channel (self, handle, base_conn->self_handle, TRUE);
+      channel = _muc_manager_new_channel (self, handle,
+          tp_base_connection_get_self_handle (base_conn), TRUE);
       idle_muc_channel_join_attempt (channel);
     }
 
